@@ -1,9 +1,10 @@
-'''
-This file is part of khmer, http://github.com/ged-lab/khmer/, and is
-Copyright (C) Michigan State University, 2009-2014. It is licensed under
-the three-clause BSD license; see doc/LICENSE.txt.
-Contact: khmer-project@idyll.org
-'''
+#
+# This file is part of khmer, http://github.com/ged-lab/khmer/, and is
+# Copyright (C) Michigan State University, 2014. It is licensed under
+# the three-clause BSD license; see doc/LICENSE.txt.
+# Contact: khmer-project@idyll.org
+#
+
 import sys
 import os
 import argparse
