@@ -1,7 +1,8 @@
 //
 // This file is part of khmer, http://github.com/ged-lab/khmer/, and is
 // Copyright (C) Michigan State University, 2009-2013. It is licensed under
-// the three-clause BSD license; see doc/LICENSE.txt. Contact: ctb@msu.edu
+// the three-clause BSD license; see doc/LICENSE.txt.
+// Contact: khmer-project@idyll.org
 //
 
 #include "aligner.hh"
@@ -53,7 +54,7 @@ std::vector<Node*>::iterator node_vector_find(std::vector<Node*>& a,
 }
 
 Node * Aligner::subalign(Node * startVert,
-                         unsigned int seqLen,
+                         unsigned long seqLen,
                          unsigned char forward,
                          std::set<Node*>& closed,
                          std::vector<Node*>& open,
@@ -105,7 +106,7 @@ Node * Aligner::subalign(Node * startVert,
 
 std::string Aligner::extractString(Node* goal,
                                    unsigned char forward,
-                                   std::map<int,int>* readDeletions)
+                                   std::map<unsigned long,unsigned long>* readDeletions)
 {
     std::string ret;
 
@@ -130,7 +131,7 @@ std::string Aligner::extractString(Node* goal,
     if (forward) {
         std::string tmp;
 
-        for (int i = ret.length()-1; i >= 0; i--) {
+        for (long long i = ret.length()-1; i >= 0; i--) {
             tmp += ret[i];
         }
 
@@ -180,7 +181,7 @@ CandidateAlignment Aligner::align(CountingHash * ch,
         return CandidateAlignment();
     }
 
-    std::map<int,int> readDels;
+    std::map<unsigned long,unsigned long> readDels;
 
     std::string align = extractString(leftGoal, 0, &readDels) +
                         kmer +
@@ -222,9 +223,11 @@ void Aligner::printErrorFootprint(const std::string& read)
     for (unsigned int i = 0; i < read.length() - k + 1; i++) {
         std::string kmer = read.substr(i, k);
 
-        assert(kmer.length() == k);
+        if (!(kmer.length() == k)) {
+            throw std::exception();
+        }
 
-        int kCov = ch->get_count(kmer.c_str());
+        BoundedCounterType kCov = ch->get_count(kmer.c_str());
 
         bool isCorrect = isCorrectKmer(kCov, lambdaOne, lambdaTwo);
 
@@ -242,7 +245,7 @@ CandidateAlignment Aligner::alignRead(const std::string& read)
     unsigned int longestErrorRegion = 0;
     unsigned int currentErrorRegion = 0;
 
-    unsigned int k = ch->ksize();
+    WordLength k = ch->ksize();
 
     std::set<CandidateAlignment> alignments;
     CandidateAlignment best = CandidateAlignment();
@@ -252,9 +255,11 @@ CandidateAlignment Aligner::alignRead(const std::string& read)
     for (unsigned int i = 0; i < read.length() - k + 1; i++) {
         std::string kmer = read.substr(i, k);
 
-        assert(kmer.length() == k);
+        if (!(kmer.length() == k)) {
+            throw std::exception();
+        }
 
-        int kCov = ch->get_count(kmer.c_str());
+        BoundedCounterType kCov = ch->get_count(kmer.c_str());
 
         bool isCorrect = isCorrectKmer(kCov, lambdaOne, lambdaTwo);
 
@@ -296,7 +301,7 @@ CandidateAlignment Aligner::alignRead(const std::string& read)
 
     // read appears to be error free
     if (markers.size() == 1 && markers[0] == 0) {
-        std::map<int,int> readDels;
+        std::map<unsigned long,unsigned long> readDels;
         CandidateAlignment retAln = CandidateAlignment(readDels, read);
         return retAln;
     }
@@ -354,7 +359,7 @@ CandidateAlignment Aligner::alignRead(const std::string& read)
         }
     }
 
-    std::map<int,int> readDels;
+    std::map<unsigned long,unsigned long> readDels;
     CandidateAlignment retAln = CandidateAlignment(readDels, graphAlign);
     return retAln;
 }
