@@ -1,7 +1,8 @@
 //
 // This file is part of khmer, http://github.com/ged-lab/khmer/, and is
 // Copyright (C) Michigan State University, 2009-2013. It is licensed under
-// the three-clause BSD license; see doc/LICENSE.txt. Contact: ctb@msu.edu
+// the three-clause BSD license; see doc/LICENSE.txt.
+// Contact: khmer-project@idyll.org
 //
 
 #include "node.hh"
@@ -106,12 +107,11 @@ Kmer Node::makeNextKmer(unsigned char forward, char b)
 
 Node::Node(Node* _parent,
            char _emission,
-           unsigned int _stateNo,
+           unsigned long _stateNo,
            char _state,
-           Kmer _kmer)
+           Kmer _kmer) : kmer( _kmer)
 {
     parent = _parent;
-    kmer = _kmer;
     emission = _emission;
     stateNo = _stateNo;
     state = _state;
@@ -137,8 +137,8 @@ std::queue<Node*> Node::enumerate(CountingHash* ch,
 {
     std::queue<Node*> ret;
 
-    int index;
-    int remaining;
+    size_t index;
+    size_t remaining;
     double bestMatch = sm->score('A','A');
     double errorOffset = 20.0;
 
