@@ -1,3 +1,10 @@
+//
+// This file is part of khmer, http://github.com/ged-lab/khmer/, and is
+// Copyright (C) Michigan State University, 2013. It is licensed under
+// the three-clause BSD license; see doc/LICENSE.txt.
+// Contact: khmer-project@idyll.org
+//
+
 #include "khmer.hh"
 #include "hashtable.hh"
 #include "hashbits.hh"
@@ -6,7 +13,8 @@
 
 using namespace khmer;
 
-int main() {
+int main()
+{
     HashIntoType sizes[] = { 100000003, 100000004, 100000007, 10000000011};
     std::vector<HashIntoType> sizes_vec (sizes, sizes + sizeof(sizes) / sizeof(HashIntoType) );
 
@@ -15,6 +23,6 @@ int main() {
 
     std::cout << "lh_pointer n_tags: " << lh_pointer->n_tags() << std::endl;
     std::cout << "hb_pointer n_tags: " << hb_pointer->n_tags() << std::endl;
-    
+
     return 0;
 }
