@@ -9,6 +9,7 @@
 """
 Take a list of files containing sequences, and subsample 100,000 sequences (-N)
 uniformly, using reservoir sampling.  Stop after first 100m sequences (-M).
+By default take one subsample, but take -S samples if specified.
 
 % scripts/sample-reads-randomly.py <infile>
 
@@ -19,6 +20,9 @@ import argparse
 import screed
 import os.path
 import random
+import textwrap
+import sys
+
 import khmer
 from khmer.file import check_file_status, check_space
 from khmer.khmer_args import info
@@ -29,17 +33,37 @@ DEBUG = True
 
 
 def get_parser():
+    epilog = ("""
+    
+    Take a list of files containing sequences, and subsample 100,000
+    sequences (:option:`-N`/:option:`--num_reads`) uniformly, using
+    reservoir sampling.  Stop after first 100m sequences
+    (:option:`-M`/:option:`--max_reads`). By default take one subsample,
+    but take :option:`-S`/:option:`--samples` samples if specified.
+    
+    The output is placed in :option:`-o`/:option:`--output` <file>
+    (for a single sample) or in <file>.subset.0 to <file>.subset.S-1
+    (for more than one sample).
+
+    This script uses the `reservoir sampling
+    <http://en.wikipedia.org/wiki/Reservoir_sampling>`__ algorithm.
+    """)   # noqa
+
     parser = argparse.ArgumentParser(
         description="Uniformly subsample sequences from a collection of files",
-        formatter_class=argparse.ArgumentDefaultsHelpFormatter)
+        formatter_class=argparse.ArgumentDefaultsHelpFormatter,
+        epilog=textwrap.dedent(epilog))
 
     parser.add_argument('filenames', nargs='+')
     parser.add_argument('-N', '--num_reads', type=int, dest='num_reads',
                         default=DEFAULT_NUM_READS)
     parser.add_argument('-M', '--max_reads', type=int, dest='max_reads',
                         default=DEFAULT_MAX_READS)
+    parser.add_argument('-S', '--samples', type=int, dest='num_samples',
+                        default=1)
     parser.add_argument('-R', '--random-seed', type=int, dest='random_seed')
     parser.add_argument('-o', '--output', dest='output_file',
+                        metavar='output_file',
                         type=argparse.FileType('w'), default=None)
     parser.add_argument('--version', action='version', version='%(prog)s '
                         + khmer.__version__)
@@ -66,23 +90,38 @@ def main():
     if args.random_seed:
         random.seed(args.random_seed)
 
+    # bound n_samples
+    num_samples = max(args.num_samples, 1)
+
     #
     # Figure out what the output filename is going to be
     #
 
     output_file = args.output_file
     if output_file:
+        if num_samples > 1:
+            sys.stderr.write(
+                "Error: cannot specify -o with more than one sample.")
+            sys.exit(-1)
         output_filename = output_file.name
     else:
         filename = args.filenames[0]
         output_filename = os.path.basename(filename) + '.subset'
-        output_file = open(output_filename, 'w')
 
-    print 'Subsampling %d reads using reservoir sampling.' % args.num_reads
-    print 'Subsampled reads will be placed in %s' % output_filename
-    print ''
+    if num_samples == 1:
+        print 'Subsampling %d reads using reservoir sampling.' % args.num_reads
+        print 'Subsampled reads will be placed in %s' % output_filename
+        print ''
+    else:  # > 1
+        print 'Subsampling %d reads, %d times, using reservoir sampling.' % \
+            (args.num_reads, num_samples)
+        print 'Subsampled reads will be placed in %s.N' % output_filename
+        print ''
 
     reads = []
+    for n in range(num_samples):
+        reads.append([])
+
     total = 0
 
     # read through all the sequences and load/resample the reservoir
@@ -100,20 +139,32 @@ def main():
 
             # collect first N reads
             if total <= args.num_reads:
-                reads.append(record)
+                for n in range(num_samples):
+                    reads[n].append(record)
             else:
                 # use reservoir sampling to replace reads at random
                 # see http://en.wikipedia.org/wiki/Reservoir_sampling
-                guess = random.randint(1, total)
-                if guess <= args.num_reads:
-                    reads[guess - 1] = record
+
+                for n in range(num_samples):
+                    guess = random.randint(1, total)
+                    if guess <= args.num_reads:
+                        reads[n][guess - 1] = record
 
     # output all the subsampled reads:
-    for record in reads:
-        output_file.write(output_single(record))
+    if len(reads) == 1:
+        print 'Writing %d sequences to %s' % (len(reads[0]), output_filename)
+        if not output_file:
+            output_file = open(output_filename, 'w')
 
-    print ''
-    print 'wrote %d reads to %s' % (len(reads), output_filename)
+        for record in reads[0]:
+            output_file.write(output_single(record))
+    else:
+        for n in range(num_samples):
+            n_filename = output_filename + '.%d' % n
+            print 'Writing %d sequences to %s' % (len(reads[n]), n_filename)
+            output_file = open(n_filename, 'w')
+            for record in reads[n]:
+                output_file.write(output_single(record))
 
 if __name__ == '__main__':
     main()
