@@ -7,15 +7,20 @@
 import tempfile
 import os
 import shutil
-
-thisdir = os.path.dirname(__file__)
-thisdir = os.path.abspath(thisdir)
-
-tempfile.tempdir = thisdir
+from pkg_resources import Requirement, resource_filename, ResolutionError
 
 
 def get_test_data(filename):
-    return os.path.join(thisdir, 'test-data', filename)
+    filepath = None
+    try:
+        filepath = resource_filename(
+            Requirement.parse("khmer"), "khmer/tests/test-data/" + filename)
+    except ResolutionError:
+        pass
+    if not filepath or not os.path.isfile(filepath):
+        filepath = os.path.join(os.path.dirname(__file__), 'test-data',
+                                filename)
+    return filepath
 
 cleanup_list = []
 
