@@ -4,6 +4,7 @@
 # the three-clause BSD license; see doc/LICENSE.txt.
 # Contact: khmer-project@idyll.org
 #
+# pylint: disable=missing-docstring
 import khmer
 import screed
 
@@ -110,3 +111,64 @@ def test_fakelump_repartitioning():
 
     (n_partitions, n_singletons) = ht.count_partitions()
     assert n_partitions == 3, n_partitions
+
+
+@attr('highmem')
+def test_fakelump_load_stop_tags_trunc():
+    fakelump_fa = utils.get_test_data('fakelump.fa')
+    fakelump_fa_foo = utils.get_temp_filename('fakelump.fa.stopfoo')
+
+    ht = khmer.new_hashbits(32, 1e7, 4)
+    ht.consume_fasta_and_tag(fakelump_fa)
+
+    subset = ht.do_subset_partition(0, 0)
+    ht.merge_subset(subset)
+
+    (n_partitions, n_singletons) = ht.count_partitions()
+    assert n_partitions == 1, n_partitions
+
+    # now, break partitions on any k-mer that you see more than once
+    # on big excursions, where big excursions are excursions 40 out
+    # that encounter more than 82 k-mers.  This should specifically
+    # identify our connected sequences in fakelump...
+
+    EXCURSION_DISTANCE = 40
+    EXCURSION_KMER_THRESHOLD = 82
+    EXCURSION_KMER_COUNT_THRESHOLD = 1
+    counting = khmer.new_counting_hash(32, 1e7, 4)
+
+    ht.repartition_largest_partition(None, counting,
+                                     EXCURSION_DISTANCE,
+                                     EXCURSION_KMER_THRESHOLD,
+                                     EXCURSION_KMER_COUNT_THRESHOLD)
+
+    ht.save_stop_tags(fakelump_fa_foo)
+    data = open(fakelump_fa_foo).read()
+
+    fp = open(fakelump_fa_foo, 'wb')
+    fp.write(data[:10])
+    fp.close()
+
+    # ok, now try loading these stop tags; should fail.
+    ht = khmer.new_hashbits(32, 1e7, 4)
+    ht.consume_fasta_and_tag(fakelump_fa)
+
+    try:
+        ht.load_stop_tags(fakelump_fa_foo)
+        assert 0, "this test should fail"
+    except IOError:
+        pass
+
+
+@attr('highmem')
+def test_fakelump_load_stop_tags_notexist():
+    fakelump_fa_foo = utils.get_temp_filename('fakelump.fa.stopfoo')
+
+    # ok, now try loading these stop tags; should fail.
+    ht = khmer.new_hashbits(32, 1e7, 4)
+
+    try:
+        ht.load_stop_tags(fakelump_fa_foo)
+        assert 0, "this test should fail"
+    except IOError:
+        pass
