@@ -1,6 +1,6 @@
 #
 # This file is part of khmer, http://github.com/ged-lab/khmer/, and is
-# Copyright (C) Michigan State University, 2009-2013. It is licensed under
+# Copyright (C) Michigan State University, 2009-2014. It is licensed under
 # the three-clause BSD license; see doc/LICENSE.txt.
 # Contact: khmer-project@idyll.org
 #
@@ -13,22 +13,40 @@ import shutil
 from cStringIO import StringIO
 import traceback
 
-import tests.khmer_tst_utils as utils
+import khmer_tst_utils as utils
 import khmer
 import khmer.file
 import screed
 
 
-def scriptpath(scriptfile):
-    path = os.path.join(utils.thisdir, '..', 'scripts', scriptfile)
-    path = os.path.abspath(path)
-    return path
+def scriptpath(script):
+    return script
 
 
 def teardown():
     utils.cleanup()
 
 
+def _runscript(scriptname):
+    import pkg_resources
+    ns = {"__name__": "__main__"}
+    ns['sys'] = globals()['sys']
+    try:
+        pkg_resources.get_distribution("khmer").run_script(
+            scriptname, ns)
+        return 0
+    except pkg_resources.ResolutionError, err:
+        paths = [os.path.join(os.path.dirname(__file__),
+                              "../scripts")]
+        paths.extend(os.environ['PATH'].split(':'))
+        for path in paths:
+            scriptfile = os.path.join(path, scriptname)
+            if os.path.isfile(scriptfile):
+                execfile(scriptfile, ns)
+                return 0
+    return -1
+
+
 def runscript(scriptname, args, in_directory=None, fail_ok=False):
     """
     Run the given Python script, with the given args, in the given directory,
@@ -40,6 +58,7 @@ def runscript(scriptname, args, in_directory=None, fail_ok=False):
     cwd = os.getcwd()
 
     try:
+        status = -1
         oldargs = sys.argv
         sys.argv = sysargs
 
@@ -53,8 +72,7 @@ def runscript(scriptname, args, in_directory=None, fail_ok=False):
         try:
             print 'running:', scriptname, 'in:', in_directory
             print 'arguments', sysargs
-            execfile(scriptname, {'__name__': '__main__'})
-            status = 0
+            status = _runscript(scriptname)
         except SystemExit, e:
             status = e.code
         except:
@@ -70,48 +88,11 @@ def runscript(scriptname, args, in_directory=None, fail_ok=False):
     if status != 0 and not fail_ok:
         print out
         print err
-        raise Exception(status, out, err)
+        assert False, (status, out, err)
 
     return status, out, err
 
 
-def DEBUG_runscript(scriptname, args, in_directory=None, fail_ok=False):
-    """
-    Run the given Python script, with the given args, in the given directory,
-    using 'execfile'.
-
-    @CTB what does this do differently from runscript?
-    """
-    sysargs = [scriptname]
-    sysargs.extend(args)
-
-    cwd = os.getcwd()
-
-    try:
-        oldargs = sys.argv
-        sys.argv = sysargs
-
-        if in_directory:
-            os.chdir(in_directory)
-
-        try:
-            print 'running:', scriptname, 'in:', in_directory
-            execfile(scriptname, {'__name__': '__main__'})
-            status = 0
-        except:
-            traceback.print_exc(file=sys.stderr)
-            status = -1
-    finally:
-        sys.argv = oldargs
-
-        os.chdir(cwd)
-
-    if status != 0 and not fail_ok:
-        raise Exception(status)
-
-    return status, "", ""
-
-
 def test_check_space():
     # @CTB this probably belongs in a new test file, along with other
     # tests of the file.py module.
@@ -120,14 +101,15 @@ def test_check_space():
 
 def test_load_into_counting():
     script = scriptpath('load-into-counting.py')
-    args = ['-x', '1e7', '-N', '2', '-k', '20']
+    args = ['-x', '1e7', '-N', '2', '-k', '20', '-t']
 
     outfile = utils.get_temp_filename('out.kh')
     infile = utils.get_test_data('test-abund-read-2.fa')
 
     args.extend([outfile, infile])
 
-    runscript(script, args)
+    (status, out, err) = runscript(script, args)
+    assert 'Total number of k-mers: 95' in err, err
     assert os.path.exists(outfile)
 
 
@@ -233,8 +215,10 @@ def test_filter_abund_1_singlefile():
     shutil.copyfile(utils.get_test_data('test-abund-read-2.fa'), infile)
 
     script = scriptpath('filter-abund-single.py')
-    args = ['-x', '1e7', '-N', '2', '-k', '17', infile]
-    runscript(script, args, in_dir)
+    args = ['-x', '1e7', '-N', '2', '-k', '17', '-t', infile]
+    (status, out, err) = runscript(script, args, in_dir)
+
+    assert 'Total number of k-mers: 98' in err, err
 
     outfile = infile + '.abundfilt'
     assert os.path.exists(outfile), outfile
@@ -353,8 +337,10 @@ def test_normalize_by_median():
     shutil.copyfile(utils.get_test_data('test-abund-read-2.fa'), infile)
 
     script = scriptpath('normalize-by-median.py')
-    args = ['-C', CUTOFF, '-k', '17', infile]
-    runscript(script, args, in_dir)
+    args = ['-C', CUTOFF, '-k', '17', '-t', infile]
+    (status, out, err) = runscript(script, args, in_dir)
+
+    assert 'Total number of k-mers: 98' in err, err
 
     outfile = infile + '.keep'
     assert os.path.exists(outfile), outfile
@@ -556,14 +542,16 @@ def test_count_median():
 
 def test_load_graph():
     script = scriptpath('load-graph.py')
-    args = ['-x', '1e7', '-N', '2', '-k', '20']
+    args = ['-x', '1e7', '-N', '2', '-k', '20', '-t']
 
     outfile = utils.get_temp_filename('out')
     infile = utils.get_test_data('random-20-a.fa')
 
     args.extend([outfile, infile])
 
-    runscript(script, args)
+    (status, out, err) = runscript(script, args)
+
+    assert 'Total number of k-mers: 3959' in err, err
 
     ht_file = outfile + '.pt'
     assert os.path.exists(ht_file), ht_file
@@ -627,7 +615,7 @@ def _make_graph(infilename, min_hashsize=1e7, n_hashes=2, ksize=20,
     args = ['-x', str(min_hashsize), '-N', str(n_hashes), '-k', str(ksize)]
 
     outfile = utils.get_temp_filename('out')
-    infile = utils.get_test_data(infilename)
+    infile = infilename
 
     args.extend([outfile, infile])
 
@@ -678,7 +666,7 @@ def _DEBUG_make_graph(infilename, min_hashsize=1e7, n_hashes=2, ksize=20,
 
     args.extend([outfile, infile])
 
-    DEBUG_runscript(script, args)
+    runscript(script, args)
 
     ht_file = outfile + '.ct'
     assert os.path.exists(ht_file), ht_file
@@ -692,12 +680,12 @@ def _DEBUG_make_graph(infilename, min_hashsize=1e7, n_hashes=2, ksize=20,
         args = [outfile]
         if stop_big_traverse:
             args.insert(0, '--no-big-traverse')
-        DEBUG_runscript(script, args)
+        runscript(script, args)
 
         print ">>>> DEBUG: Merging Partitions <<<"
         script = scriptpath('merge-partitions.py')
         args = [outfile, '-k', str(ksize)]
-        DEBUG_runscript(script, args)
+        runscript(script, args)
 
         final_pmap_file = outfile + '.pmap.merged'
         assert os.path.exists(final_pmap_file)
@@ -708,7 +696,7 @@ def _DEBUG_make_graph(infilename, min_hashsize=1e7, n_hashes=2, ksize=20,
             args = ["-k", str(ksize), outfile, infilename]
 
             in_dir = os.path.dirname(outfile)
-            DEBUG_runscript(script, args, in_dir)
+            runscript(script, args, in_dir)
 
             baseinfile = os.path.basename(infilename)
             assert os.path.exists(os.path.join(in_dir, baseinfile + '.part'))
@@ -1098,8 +1086,11 @@ def test_abundance_dist_single():
     shutil.copyfile(utils.get_test_data('test-abund-read-2.fa'), infile)
 
     script = scriptpath('abundance-dist-single.py')
-    args = ['-x', '1e7', '-N', '2', '-k', '17', '-z', infile, outfile]
-    runscript(script, args, in_dir)
+    args = ['-x', '1e7', '-N', '2', '-k', '17', '-z', '-t', infile,
+            outfile]
+    (status, out, err) = runscript(script, args, in_dir)
+
+    assert 'Total number of k-mers: 98' in err, err
 
     fp = iter(open(outfile))
     line = fp.next().strip()
@@ -1378,102 +1369,132 @@ def test_sample_reads_randomly():
                         '895:1:1:1335:19932', '895:1:1:1340:19387'])
 
 
-def test_sweep_reads_buffered():
-    readfile = utils.get_temp_filename('reads.fa')
-    contigfile = utils.get_temp_filename('contigs.fp')
-    in_dir = os.path.dirname(contigfile)
+def test_fastq_to_fasta():
 
-    shutil.copyfile(utils.get_test_data('test-sweep-reads.fa'), readfile)
-    shutil.copyfile(utils.get_test_data('test-sweep-contigs.fp'), contigfile)
+    script = scriptpath('fastq-to-fasta.py')
+    clean_infile = utils.get_temp_filename('test-clean.fq')
+    n_infile = utils.get_temp_filename('test-n.fq')
 
-    script = scriptpath('sweep-reads-buffered.py')
-    args = ['-k', '25', '--prefix', 'test', '--label-by-pid',
-            contigfile, readfile, 'junkfile.fa']
+    shutil.copyfile(utils.get_test_data('test-fastq-reads.fq'), clean_infile)
+    shutil.copyfile(utils.get_test_data('test-fastq-n-reads.fq'), n_infile)
 
-    status, out, err = runscript(script, args, in_dir, fail_ok=True)
+    clean_outfile = clean_infile + '.keep.fa'
+    n_outfile = n_infile + '.keep.fa'
 
-    # check if the bad file was skipped without issue
-    assert 'ERROR' in err, err
-    assert 'skipping' in err, err
+    in_dir = os.path.dirname(clean_infile)
+    in_dir_n = os.path.dirname(n_infile)
 
-    out1 = os.path.join(in_dir, 'test_0.fa')
-    out2 = os.path.join(in_dir, 'test_1.fa')
-    mout = os.path.join(in_dir, 'test_multi.fa')
-    oout = os.path.join(in_dir, 'test_orphaned.fa')
+    args = [clean_infile, '-n', '-o', clean_outfile]
+    (status, out, err) = runscript(script, args, in_dir)
+    assert len(out.splitlines()) == 2, len(out.splitlines())
+    assert "No lines dropped" in err
 
-    print os.listdir(in_dir)
+    args = [n_infile, '-n', '-o', n_outfile]
+    (status, out, err) = runscript(script, args, in_dir_n)
+    assert len(out.splitlines()) == 2
+    assert "No lines dropped" in err
 
-    seqs1 = set([r.name for r in screed.open(out1)])
-    seqs2 = set([r.name for r in screed.open(out2)])
-    seqsm = set([r.name for r in screed.open(mout)])
-    seqso = set([r.name for r in screed.open(oout)])
+    args = [clean_infile, '-o', clean_outfile]
+    (status, out, err) = runscript(script, args, in_dir)
+    assert len(out.splitlines()) == 2
+    assert "0 lines dropped" in err
 
-    print seqs1
-    print seqs2
-    print seqsm
-    print seqso
-    assert seqs1 == set(['read1_p0\t0', 'read2_p0\t0'])
-    assert seqs2 == set(['read3_p1\t1'])
-    assert (seqsm == set(['read4_multi\t0\t1']) or
-            seqsm == set(['read4_multi\t1\t0']))
-    assert seqso == set(['read5_orphan'])
+    args = [n_infile, '-o', n_outfile]
+    (status, out, err) = runscript(script, args, in_dir_n)
+    assert len(out.splitlines()) == 2, out
+    assert "4 lines dropped" in err, err
 
-    try:
-        os.remove(out1)
-        os.remove(out2)
-        os.remove(mout)
-        os.remove(oout)
-    except (IOError, OSError) as e:
-        print >>sys.stderr, 'error removing test outputs'
+    args = [clean_infile]
+    (status, out, err) = runscript(script, args, in_dir)
+    assert len(out.splitlines()) > 2
+    assert "0 lines dropped" in err
+
+    args = [n_infile]
+    (status, out, err) = runscript(script, args, in_dir_n)
+    assert len(out.splitlines()) > 2
+    assert "4 lines dropped" in err
 
 
-def test_sweep_reads_buffered_2():
+def test_extract_long_sequences():
 
-    infile = utils.get_temp_filename('seqs.fa')
-    inref = utils.get_temp_filename('ref.fa')
-    shutil.copyfile(utils.get_test_data('random-20-X2.fa'), infile)
-    shutil.copyfile(utils.get_test_data('random-20-a.fa'), inref)
-    wdir = os.path.dirname(inref)
-    script = scriptpath('sweep-reads-buffered.py')
-    args = ['-m', '50', '-k', '20', '-l', '9', '-b', '60', '--prefix',
-            'test', '--label-by-seq', inref, infile]
-    status, out, err = runscript(script, args, wdir)
+    script = scriptpath('extract-long-sequences.py')
+    fq_infile = utils.get_temp_filename('test.fq')
+    fa_infile = utils.get_temp_filename('test.fa')
 
-    for i in xrange(99):
-        p = os.path.join(wdir, 'test_{i}.fa'.format(i=i))
-        print p, err, out
-        assert os.path.exists(p)
-        os.remove(p)
-    assert os.path.exists(os.path.join(wdir, 'test.counts.csv'))
-    assert os.path.exists(os.path.join(wdir, 'test.dist.txt'))
-    assert not os.path.exists(os.path.join(wdir, 'test_multi.fa'))
+    shutil.copyfile(utils.get_test_data('paired-mixed.fq'), fq_infile)
+    shutil.copyfile(utils.get_test_data('paired-mixed.fa'), fa_infile)
+
+    fq_outfile = fq_infile + '.keep.fq'
+    fa_outfile = fa_infile + '.keep.fa'
+
+    in_dir_fq = os.path.dirname(fq_infile)
+    in_dir_fa = os.path.dirname(fa_infile)
+
+    args = [fq_infile, '-l', '10', '-o', 'fq_outfile']
+    (status, out, err) = runscript(script, args, in_dir_fa)
+
+    countlines = sum(1 for line in open(fq_infile))
+    assert countlines == 44, countlines
+
+    args = [fa_infile, '-l', '10', '-o', 'fa_outfile']
+    (status, out, err) = runscript(script, args, in_dir_fa)
+
+    countlines = sum(1 for line in open(fa_infile))
+    assert countlines == 22, countlines
 
 
-def test_sweep_reads_buffered_3():
+def test_sample_reads_randomly_S():
+    infile = utils.get_temp_filename('test.fq')
+    in_dir = os.path.dirname(infile)
 
-    infile = utils.get_temp_filename('seqs.fa')
-    shutil.copyfile(utils.get_test_data('random-20-a.fa'), infile)
-    wdir = os.path.dirname(infile)
-    script = scriptpath('sweep-reads-buffered.py')
-    args = ['-m', '75', '-k', '20', '-l', '1', '--prefix',
-            'test', '--label-by-group', '10', infile, infile]
-    status, out, err = runscript(script, args, wdir)
+    shutil.copyfile(utils.get_test_data('test-fastq-reads.fq'), infile)
 
-    for i in xrange(10):
-        p = os.path.join(wdir, 'test_{i}.fa'.format(i=i))
-        print p, err, out
-        assert os.path.exists(p)
-        os.remove(p)
+    script = scriptpath('sample-reads-randomly.py')
 
-    counts_fn = os.path.join(wdir, 'test.counts.csv')
-    with open(counts_fn) as cfp:
-        for line in cfp:
-            _, _, c = line.partition(',')
-            assert int(c) in [9, 10]
+    # fix random number seed for reproducibility
+    args = ['-N', '10', '-R', '1', '-S', '3']
 
-    assert os.path.exists(counts_fn)
-    assert os.path.exists(os.path.join(wdir, 'test.dist.txt'))
-    assert not os.path.exists(os.path.join(wdir, 'test_multi.fa'))
+    badargs = list(args)
+    badargs.extend(['-o', 'test', 'test.fq', 'test.fq'])
+    (status, out, err) = runscript(script, badargs, in_dir, fail_ok=True)
+    assert status == -1, (status, out, err)
+
+    args.append('test.fq')
+
+    runscript(script, args, in_dir)
+
+    outfile = infile + '.subset.0'
+    assert os.path.exists(outfile), outfile
+
+    seqs = set([r.name for r in screed.open(outfile)])
+    print seqs
+    assert seqs == set(['895:1:1:1298:13380', '895:1:1:1347:3237',
+                        '895:1:1:1295:6189', '895:1:1:1342:11001',
+                        '895:1:1:1252:19493', '895:1:1:1318:10532',
+                        '895:1:1:1314:10430', '895:1:1:1347:8723',
+                        '895:1:1:1381:4958', '895:1:1:1338:6614'])
+
+    outfile = infile + '.subset.1'
+    assert os.path.exists(outfile), outfile
+
+    seqs = set([r.name for r in screed.open(outfile)])
+    print seqs
+    assert seqs == set(['895:1:1:1384:20217', '895:1:1:1347:3237',
+                        '895:1:1:1348:18672', '895:1:1:1290:11501',
+                        '895:1:1:1386:7536', '895:1:1:1373:13994',
+                        '895:1:1:1355:13535', '895:1:1:1303:6251',
+                        '895:1:1:1381:4958', '895:1:1:1338:6614'])
+
+    outfile = infile + '.subset.2'
+    assert os.path.exists(outfile), outfile
+
+    seqs = set([r.name for r in screed.open(outfile)])
+    print seqs
+    assert seqs == set(['895:1:1:1326:7273', '895:1:1:1384:20217',
+                        '895:1:1:1347:3237', '895:1:1:1353:6642',
+                        '895:1:1:1340:19387', '895:1:1:1252:19493',
+                        '895:1:1:1381:7062', '895:1:1:1383:3089',
+                        '895:1:1:1342:20695', '895:1:1:1303:6251'])
 
 
 def test_count_overlap():
