@@ -4,6 +4,7 @@
 # the three-clause BSD license; see doc/LICENSE.txt.
 # Contact: khmer-project@idyll.org
 #
+# pylint: disable=missing-docstring
 import khmer
 import screed
 
@@ -18,7 +19,7 @@ def teardown():
 class Test_RandomData(object):
 
     def test_3_merge_013(self):
-        ht = khmer.new_hashbits(20, 4 ** 14 + 1)
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
 
         filename = utils.get_test_data('test-graph2.fa')
 
@@ -38,7 +39,7 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_3_merge_023(self):
-        ht = khmer.new_hashbits(20, 4 ** 14 + 1)
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
         filename = utils.get_test_data('test-graph2.fa')
 
         (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
@@ -57,7 +58,7 @@ class Test_RandomData(object):
         assert n_partitions == 1, n_partitions        # combined.
 
     def test_5_merge_046(self):
-        ht = khmer.new_hashbits(20, 4 ** 14 + 1)
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
         filename = utils.get_test_data('test-graph5.fa')
 
         (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
@@ -194,7 +195,7 @@ class Test_SaveLoadPmap(object):
 
     @attr('highmem')
     def test_save_load_merge(self):
-        ht = khmer.new_hashbits(20, 4 ** 14 + 1)
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
         filename = utils.get_test_data('test-graph2.fa')
 
         (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
@@ -227,7 +228,7 @@ class Test_SaveLoadPmap(object):
 
     @attr('highmem')
     def test_save_load_merge_2(self):
-        ht = khmer.new_hashbits(20, 4 ** 14 + 1)
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
         filename = utils.get_test_data('random-20-a.fa')
 
         (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
@@ -256,9 +257,17 @@ class Test_SaveLoadPmap(object):
         n_partitions = ht.output_partitions(filename, outfile)
         assert n_partitions == 1, n_partitions        # combined.
 
+    def test_save_load_merge_nexist(self):
+        ht = khmer.new_hashbits(20, 1)
+        try:
+            a = ht.load_subset_partitionmap('this does not exist')
+            assert 0, "this should not succeed"
+        except IOError, e:
+            print str(e)
+
     @attr('highmem')
     def test_save_merge_from_disk(self):
-        ht = khmer.new_hashbits(20, 4 ** 14 + 1)
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
         filename = utils.get_test_data('test-graph2.fa')
 
         (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
@@ -288,7 +297,7 @@ class Test_SaveLoadPmap(object):
 
     @attr('highmem')
     def test_save_merge_from_disk_2(self):
-        ht = khmer.new_hashbits(20, 4 ** 14 + 1)
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
         filename = utils.get_test_data('random-20-a.fa')
 
         (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
@@ -314,6 +323,73 @@ class Test_SaveLoadPmap(object):
         n_partitions = ht.output_partitions(filename, outfile)
         assert n_partitions == 1, n_partitions        # combined.
 
+    @attr('highmem')
+    def test_save_merge_from_disk_file_not_exist(self):
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
+        filename = utils.get_test_data('test-graph2.fa')
+
+        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        assert total_reads == 3, total_reads
+
+        divvy = ht.divide_tags_into_subsets(1)
+        print divvy
+        (a, b, c) = divvy
+
+        outfile1 = utils.get_temp_filename('x.pmap')
+
+        # fail to create file... => failure expected
+
+        try:
+            ht.merge_subset_from_disk(outfile1)
+            assert 0, "this should fail"
+        except IOError, e:
+            print str(e)
+
+    @attr('highmem')
+    def test_merge_from_disk_file_bad_type(self):
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
+        infile = utils.get_test_data('goodversion-k12.ht')
+
+        try:
+            ht.merge_subset_from_disk(infile)
+            assert 0, "this should fail"
+        except IOError, e:
+            print str(e)
+
+    @attr('highmem')
+    def test_merge_from_disk_file_version(self):
+        ht = khmer.new_hashbits(20, 4 ** 10 + 1)
+        infile = utils.get_test_data('badversion-k12.ht')
+
+        try:
+            ht.merge_subset_from_disk(infile)
+            assert 0, "this should fail"
+        except IOError, e:
+            print str(e)
+
+    def test_save_merge_from_disk_ksize(self):
+        ht = khmer.new_hashbits(20, 4 ** 6 + 1)
+        filename = utils.get_test_data('test-graph2.fa')
+
+        (total_reads, total_kmers) = ht.consume_fasta_and_tag(filename)
+        assert total_reads == 3, total_reads
+
+        divvy = ht.divide_tags_into_subsets(1)
+        print divvy
+        (a, b, c) = divvy
+
+        outfile1 = utils.get_temp_filename('x.pmap')
+        x = ht.do_subset_partition(a, b)
+        ht.save_subset_partitionmap(x, outfile1)
+        del x
+
+        ht = khmer.new_hashbits(19, 1, 1)
+        try:
+            ht.merge_subset_from_disk(outfile1)
+            assert 0, "this should fail"
+        except IOError, e:
+            print str(e)
+
 
 def test_output_partitions():
     filename = utils.get_test_data('test-output-partitions.fa')
